@@ -33,14 +33,19 @@
 #include "cleanup_task.h"
 #include "lookup_reader.h"
 #include "raw_read_iter.h"
+#include "temp_dir.h"
 
 #include <kapp/main.h>
 #include <kapp/args.h>
+
+#include <kfg/config.h> /* KConfigSetNgcFile */
+
 #include <klib/out.h>
 #include <klib/printf.h>
 #include <search/grep.h>
 #include <kfs/directory.h>
 #include <kproc/procmgr.h>
+#include <vdb/manager.h>
 
 #include <stdio.h>
 #include <os-native.h>
@@ -151,6 +156,13 @@ static const char * table_usage[] = { "which seq-table to use in case of pacbio"
 static const char * strict_usage[] = { "terminate on invalid read", NULL };
 #define OPTION_STRICT   "strict"
 
+static const char * append_usage[] = { "append to output-file", NULL };
+#define OPTION_APPEND   "append"
+#define ALIAS_APPEND    "A"
+
+static const char * ngc_usage[] = { "PATH to ngc file", NULL };
+#define OPTION_NGC   "ngc"
+
 OptDef ToolOptions[] =
 {
     { OPTION_FORMAT,    ALIAS_FORMAT,    NULL, format_usage,     1, true,   false },
@@ -179,7 +191,9 @@ OptDef ToolOptions[] =
     { OPTION_MINRDLEN,  ALIAS_MINRDLEN,  NULL, min_rl_usage,     1, true,   false },
     { OPTION_TABLE,     NULL,            NULL, table_usage,      1, true,   false },
     { OPTION_STRICT,    NULL,            NULL, strict_usage,     1, false,  false },
-    { OPTION_BASE_FLT,  ALIAS_BASE_FLT,  NULL, base_flt_usage,   10, true,  false }
+    { OPTION_BASE_FLT,  ALIAS_BASE_FLT,  NULL, base_flt_usage,   10, true,  false },
+    { OPTION_APPEND,    ALIAS_APPEND,    NULL, append_usage,     1, false,  false },
+    { OPTION_NGC,       NULL,            NULL, ngc_usage, 1, true,  false },
 };
 
 const char UsageDefaultName[] = "fasterq-dump";
@@ -211,8 +225,19 @@ rc_t CC Usage ( const Args * args )
 
     KOutMsg( "Options:\n" );
     for ( idx = 1; idx < count; ++idx ) /* start with 1, do not advertize row-range-option*/
-        HelpOptionLine( ToolOptions[ idx ] . aliases, ToolOptions[ idx ] . name, NULL, ToolOptions[ idx ] . help );
+    {
+        const OptDef * opt = &ToolOptions[idx];
+
+        const char * param = NULL;
+
+        assert(opt);
+        if (strcmp(opt->name, OPTION_NGC) == 0)
+            param = "PATH";
+
+        HelpOptionLine(opt->aliases, opt->name, param, opt->help);
+    }
     
+    KOutMsg("\n");
     HelpOptionsStandard();
     HelpVersion( fullpath, KAppVersion() );
     return rc;
@@ -220,41 +245,40 @@ rc_t CC Usage ( const Args * args )
 
 /* -------------------------------------------------------------------------------------------- */
 
-#define HOSTNAMELEN 64
 #define DFLT_PATH_LEN 4096
 
 typedef struct tool_ctx_t
 {
     KDirectory * dir;
+    const VDBManager * vdb_mgr;     /* created, but unused to avoid race-condition in threads */
 
+    const char * requested_temp_path;
     const char * accession_path;
     const char * accession_short;
     const char * output_filename;
     const char * output_dirname;
     const char * seq_tbl_name;
     
-    tmp_id tmp_id;
-
-    char hostname[ HOSTNAMELEN ];
+    struct temp_dir * temp_dir; /* temp_dir.h */
+    
     char lookup_filename[ DFLT_PATH_LEN ];
     char index_filename[ DFLT_PATH_LEN ];
     char dflt_output[ DFLT_PATH_LEN ];
     
-    struct KFastDumpCleanupTask * cleanup_task;
+    struct KFastDumpCleanupTask * cleanup_task; /* cleanup_task.h */
     
     size_t cursor_cache, buf_size, mem_limit;
 
-    uint32_t num_threads/*, max_fds */;
+    uint32_t num_threads /*, max_fds */;
     uint64_t total_ram;
     
-    format_t fmt;
+    format_t fmt; /* helper.h */
 
-    compress_t compress;    
+    compress_t compress; /* helper.h */ 
 
-    bool remove_temp_path, print_to_stdout, force;
-    bool show_progress, show_details;
+    bool force, show_progress, show_details, append, stdout;
     
-    join_options join_options;
+    join_options join_options; /* helper.h */
 } tool_ctx_t;
 
 /* taken form libs/kapp/main-priv.h */
@@ -262,40 +286,15 @@ rc_t KAppGetTotalRam ( uint64_t * totalRam );
 
 static rc_t get_environment( tool_ctx_t * tool_ctx )
 {
-    struct KProcMgr * proc_mgr;
-    rc_t rc = KProcMgrMakeSingleton ( &proc_mgr );
+    rc_t rc = KAppGetTotalRam ( &( tool_ctx -> total_ram ) );
     if ( rc != 0 )
-        ErrMsg( "cannot access process-manager" );
-    else
-    {
-        rc = KProcMgrGetPID ( proc_mgr, & tool_ctx -> tmp_id . pid );
-        if ( rc == 0 )
-        {
-            rc = KProcMgrGetHostName ( proc_mgr, tool_ctx -> hostname, sizeof tool_ctx -> hostname );
-            if ( rc != 0 )
-            {
-                size_t num_writ;
-                rc = string_printf( tool_ctx -> hostname, sizeof tool_ctx -> hostname, &num_writ, "host" );
-            }
-        }
-        if ( rc == 0 )
-            tool_ctx -> tmp_id . hostname = ( const char * )&( tool_ctx -> hostname );
-
-        KProcMgrRelease ( proc_mgr );
-    }
-    if ( rc == 0 )
-    {
-        rc = KAppGetTotalRam ( &( tool_ctx -> total_ram ) );
-        if ( rc != 0 )
-            ErrMsg( "KAppGetTotalRam() -> %R", rc );
-    }
+        ErrMsg( "KAppGetTotalRam() -> %R", rc );
     if ( rc == 0 )
     {
         rc = KDirectoryNativeDir( &( tool_ctx -> dir ) );
         if ( rc != 0 )
             ErrMsg( "KDirectoryNativeDir() -> %R", rc );
     }
-    
     return rc;
 }
 
@@ -309,7 +308,7 @@ static rc_t show_details( tool_ctx_t * tool_ctx )
     if ( rc == 0 )
         rc = KOutMsg( "threads      : %d\n", tool_ctx -> num_threads );
     if ( rc == 0 )
-        rc = KOutMsg( "scratch-path : '%s'\n", tool_ctx -> tmp_id . temp_path );
+        rc = KOutMsg( "scratch-path : '%s'\n", get_temp_dir( tool_ctx -> temp_dir ) );
     if ( rc == 0 )
         rc = KOutMsg( "output-format: " );
     if ( rc == 0 )
@@ -325,14 +324,16 @@ static rc_t show_details( tool_ctx_t * tool_ctx )
         }
     }
     if ( rc == 0 )
-    {
         rc = KOutMsg( "output-file  : '%s'\n", tool_ctx -> output_filename );
+    if ( rc == 0 )    
         rc = KOutMsg( "output-dir   : '%s'\n", tool_ctx -> output_dirname );
-    }
+    if ( rc == 0 )
+        rc = KOutMsg( "append-mode  : '%s'\n", tool_ctx -> append ? "YES" : "NO" );
+    if ( rc == 0 )
+        rc = KOutMsg( "stdout-mode  : '%s'\n", tool_ctx -> append ? "YES" : "NO" );
     return rc;
 }
 
-static const char * dflt_temp_path = "./fast.tmp";
 static const char * dflt_seq_tabl_name = "SEQUENCE";
 
 #define DFLT_CUR_CACHE ( 5 * 1024 * 1024 )
@@ -351,10 +352,8 @@ static void get_user_input( tool_ctx_t * tool_ctx, const Args * args )
     tool_ctx -> cursor_cache = get_size_t_option( args, OPTION_CURCACHE, DFLT_CUR_CACHE );            
     tool_ctx -> show_progress = get_bool_option( args, OPTION_PROGRESS );
     tool_ctx -> show_details = get_bool_option( args, OPTION_DETAILS );
-    tool_ctx -> tmp_id . temp_path = get_str_option( args, OPTION_TEMP, NULL );
-    tool_ctx -> print_to_stdout = get_bool_option( args, OPTION_STDOUT );
+    tool_ctx -> requested_temp_path = get_str_option( args, OPTION_TEMP, NULL );
     tool_ctx -> force = get_bool_option( args, OPTION_FORCE );        
-    tool_ctx -> remove_temp_path = false;
     tool_ctx -> output_filename = get_str_option( args, OPTION_OUTPUT_F, NULL );
     tool_ctx -> output_dirname = get_str_option( args, OPTION_OUTPUT_D, NULL );
     tool_ctx -> buf_size = get_size_t_option( args, OPTION_BUFSIZE, DFLT_BUF_SIZE );
@@ -380,6 +379,14 @@ static void get_user_input( tool_ctx_t * tool_ctx, const Args * args )
         tool_ctx -> join_options . skip_tech = true;
 
     tool_ctx -> seq_tbl_name = get_str_option( args, OPTION_TABLE, dflt_seq_tabl_name );
+    tool_ctx -> append = get_bool_option( args, OPTION_APPEND );
+    tool_ctx -> stdout = get_bool_option( args, OPTION_STDOUT );
+
+    {
+        const char * ngc = get_str_option(args, OPTION_NGC, NULL);
+        if (ngc != NULL)
+            KConfigSetNgcFile(ngc);
+    }
 }
 
 #define DFLT_MAX_FD 32
@@ -396,50 +403,27 @@ static void encforce_constrains( tool_ctx_t * tool_ctx )
 
     if ( tool_ctx -> buf_size > MAX_BUF_SIZE )
         tool_ctx -> buf_size = MAX_BUF_SIZE;
-
-    if ( tool_ctx -> print_to_stdout && tool_ctx -> show_progress )
-        tool_ctx -> show_progress = false;
-
-    if ( tool_ctx -> print_to_stdout && tool_ctx -> show_details )
-        tool_ctx -> show_details = false;
-
-    if ( tool_ctx -> print_to_stdout && 
-         ( ( tool_ctx -> fmt == ft_fastq_split_file ) || ( tool_ctx -> fmt == ft_fastq_split_3 ) ) )
-        tool_ctx -> print_to_stdout = false;
-}
-
-static rc_t handle_tmp_path( tool_ctx_t * tool_ctx )
-{
-    rc_t rc = 0;
-    uint32_t pt;
-    if ( tool_ctx -> tmp_id . temp_path == NULL )
+    
+    if ( tool_ctx -> stdout )
     {
-        /* if the user did not give us a temp-path: use the dflt-path and clean up after use */
-        tool_ctx -> tmp_id . temp_path = dflt_temp_path;
-        pt = KDirectoryPathType( tool_ctx -> dir, "%s", tool_ctx -> tmp_id . temp_path );
-        if ( pt != kptDir )
+        switch( tool_ctx -> fmt )
         {
-            rc = KDirectoryCreateDir ( tool_ctx -> dir, 0775, kcmInit, "%s", tool_ctx -> tmp_id . temp_path );
-            if ( rc != 0 )
-                ErrMsg( "scratch-path '%s' cannot be created!", tool_ctx -> tmp_id . temp_path );
-            else
-                tool_ctx -> remove_temp_path = true;
+            case ft_unknown             : break;
+            case ft_special             : break;
+            case ft_whole_spot          : break;
+            case ft_fastq_split_spot    : break;
+            case ft_fastq_split_file    : tool_ctx -> stdout = false; break;
+            case ft_fastq_split_3       : tool_ctx -> stdout = false; break;
         }
     }
-    else
+    
+    if ( tool_ctx -> stdout )
     {
-        /* if the user did give us a temp-path: try to create it if not force-options is given */
-        if ( !dir_exists( tool_ctx -> dir, "%s", tool_ctx -> tmp_id . temp_path ) )
-        {
-            KCreateMode create_mode = /* tool_ctx -> force ? kcmInit : kcmCreate; */ kcmInit;
-            rc = KDirectoryCreateDir ( tool_ctx -> dir, 0775, create_mode, "%s", tool_ctx -> tmp_id . temp_path );
-            if ( rc != 0 )
-                ErrMsg( "scratch-path '%s' cannot be created!", tool_ctx -> tmp_id . temp_path );
-        }
+        tool_ctx -> compress = ct_none;
+        //tool_ctx -> show_progress = false;
+        tool_ctx -> force = false;
+        tool_ctx -> append = false;
     }
-    if ( rc == 0 )
-        tool_ctx -> tmp_id . temp_path_ends_in_slash = ends_in_slash( tool_ctx -> tmp_id . temp_path ); /* helper.c */
-    return rc;
 }
 
 static rc_t handle_accession( tool_ctx_t * tool_ctx )
@@ -456,26 +440,21 @@ static rc_t handle_accession( tool_ctx_t * tool_ctx )
 
 static rc_t handle_lookup_path( tool_ctx_t * tool_ctx )
 {
-    size_t num_writ;
-    const tmp_id * tmp_id = &( tool_ctx -> tmp_id );
-    /* generate the full path of the lookup-table */        
-    rc_t rc = string_printf( &tool_ctx -> lookup_filename[ 0 ], sizeof tool_ctx -> lookup_filename,
-                            &num_writ,
-                            tmp_id -> temp_path_ends_in_slash ? "%s%s.%u.lookup" : "%s/%s.%u.lookup",
-                            tmp_id -> temp_path,
-                            tmp_id -> hostname,
-                            tmp_id -> pid );
+    rc_t rc = generate_lookup_filename( tool_ctx -> temp_dir,
+                                        &tool_ctx -> lookup_filename[ 0 ],
+                                        sizeof tool_ctx -> lookup_filename );
     if ( rc != 0 )
-        ErrMsg( "string_printf( lookup-tbl ) -> %R", rc );
+        ErrMsg( "fasterq-dump.c handle_lookup_path( lookup_filename ) -> %R", rc );
     else
     {
+        size_t num_writ;
         /* generate the full path of the lookup-index-table */                
         rc = string_printf( &tool_ctx -> index_filename[ 0 ], sizeof tool_ctx -> index_filename,
                             &num_writ,
                             "%s.idx",
                             &tool_ctx -> lookup_filename[ 0 ] );
         if ( rc != 0 )
-            ErrMsg( "string_printf( lookup-tbl-idx ) -> %R", rc );
+            ErrMsg( "fasterq-dump.c handle_lookup_path( index_filename ) -> %R", rc );
     }
     return rc;
 }
@@ -483,11 +462,10 @@ static rc_t handle_lookup_path( tool_ctx_t * tool_ctx )
 /* we have NO output-dir and NO output-file */
 static rc_t make_output_filename_from_accession( tool_ctx_t * tool_ctx )
 {
-    rc_t rc = 0;
     /* we DO NOT have a output-directory : build output-filename from the accession */
     /* generate the full path of the output-file, if not given */
     size_t num_writ;        
-    rc = string_printf( &tool_ctx -> dflt_output[ 0 ], sizeof tool_ctx -> dflt_output,
+    rc_t rc = string_printf( &tool_ctx -> dflt_output[ 0 ], sizeof tool_ctx -> dflt_output,
                         &num_writ,
                         "%s.fastq",
                         tool_ctx -> accession_short );
@@ -501,10 +479,9 @@ static rc_t make_output_filename_from_accession( tool_ctx_t * tool_ctx )
 /* we have an output-dir and NO output-file */
 static rc_t make_output_filename_from_dir_and_accession( tool_ctx_t * tool_ctx )
 {
-    rc_t rc = 0;
     size_t num_writ;
-    bool es = ends_in_slash( tool_ctx -> output_dirname );
-    rc = string_printf( tool_ctx -> dflt_output, sizeof tool_ctx -> dflt_output,
+    bool es = ends_in_slash( tool_ctx -> output_dirname ); /* helper.c */
+    rc_t rc = string_printf( tool_ctx -> dflt_output, sizeof tool_ctx -> dflt_output,
                         &num_writ,
                         es ? "%s%s.fastq" : "%s/%s.fastq",
                         tool_ctx -> output_dirname,
@@ -536,7 +513,7 @@ static rc_t adjust_output_filename( tool_ctx_t * tool_ctx )
 {
     rc_t rc = 0;
     /* we do have a output-filename : use it */
-    if ( dir_exists( tool_ctx -> dir, "%s", tool_ctx -> output_filename ) )
+    if ( dir_exists( tool_ctx -> dir, "%s", tool_ctx -> output_filename ) ) /* helper.c */
     {
         /* the given output-filename is an existing directory ! */
         rc = RC( rcVDB, rcNoTarg, rcConstructing, rcParam, rcInvalid );
@@ -549,10 +526,9 @@ static rc_t adjust_output_filename( tool_ctx_t * tool_ctx )
 
 static rc_t adjust_output_filename_by_dir( tool_ctx_t * tool_ctx )
 {
-    rc_t rc = 0;
     size_t num_writ;
-    bool es = ends_in_slash( tool_ctx -> output_dirname );
-    rc = string_printf( tool_ctx -> dflt_output, sizeof tool_ctx -> dflt_output,
+    bool es = ends_in_slash( tool_ctx -> output_dirname ); /* helper.c */
+    rc_t rc = string_printf( tool_ctx -> dflt_output, sizeof tool_ctx -> dflt_output,
                         &num_writ,
                         es ? "%s%s" : "%s/%s",
                         tool_ctx -> output_dirname,
@@ -582,7 +558,9 @@ static rc_t populate_tool_ctx( tool_ctx_t * tool_ctx, const Args * args )
         encforce_constrains( tool_ctx );
         get_environment( tool_ctx );
         
-        rc = handle_tmp_path( tool_ctx );
+        rc = make_temp_dir( &tool_ctx -> temp_dir,
+                          tool_ctx -> requested_temp_path,
+                          tool_ctx -> dir );
     }
     
     if ( rc == 0 )
@@ -616,12 +594,23 @@ static rc_t populate_tool_ctx( tool_ctx_t * tool_ctx, const Args * args )
     }
     
     if ( rc == 0 )
-        rc = Make_FastDump_Cleanup_Task ( &( tool_ctx -> cleanup_task ) );
+        rc = Make_FastDump_Cleanup_Task ( &( tool_ctx -> cleanup_task ) ); /* cleanup_task.c */
+    if ( rc == 0 )
+        rc = Add_Directory_to_Cleanup_Task ( tool_ctx -> cleanup_task, 
+                get_temp_dir( tool_ctx -> temp_dir ) );
+                
+    if ( rc == 0 )
+    {
+        rc = VDBManagerMakeRead( &( tool_ctx -> vdb_mgr ), tool_ctx -> dir );
+        if ( rc != 0 )
+            ErrMsg( "fasterq-dump.c populate_tool_ctx().VDBManagerMakeRead() -> %R\n", rc );
+    }
     return rc;
 }
 
 static rc_t print_stats( const join_stats * stats )
 {
+    KOutHandlerSetStdErr();
     rc_t rc = KOutMsg( "spots read      : %,lu\n", stats -> spots_read );
     if ( rc == 0 )
          rc = KOutMsg( "reads read      : %,lu\n", stats -> reads_read );
@@ -635,6 +624,7 @@ static rc_t print_stats( const join_stats * stats )
          rc = KOutMsg( "reads too short : %,lu\n", stats -> reads_too_short );
     if ( rc == 0 && stats -> reads_invalid > 0 )
          rc = KOutMsg( "reads invalid   : %,lu\n", stats -> reads_invalid );
+    KOutHandlerSetStdOut();
     return rc;
 }
 
@@ -664,7 +654,7 @@ static rc_t produce_lookup_files( tool_ctx_t * tool_ctx )
     if ( rc == 0 )            
         rc = make_background_file_merger( &bg_file_merger,
                                 tool_ctx -> dir,
-                                & tool_ctx -> tmp_id,
+                                tool_ctx -> temp_dir,
                                 tool_ctx -> cleanup_task,
                                 &tool_ctx -> lookup_filename[ 0 ],
                                 &tool_ctx -> index_filename[ 0 ],
@@ -678,7 +668,7 @@ static rc_t produce_lookup_files( tool_ctx_t * tool_ctx )
     if ( rc == 0 )
         rc = make_background_vector_merger( &bg_vec_merger,
                  tool_ctx -> dir,
-                 &( tool_ctx -> tmp_id ),
+                 tool_ctx -> temp_dir,
                  tool_ctx -> cleanup_task,
                  bg_file_merger,
                  tool_ctx -> num_threads,
@@ -702,6 +692,7 @@ static rc_t produce_lookup_files( tool_ctx_t * tool_ctx )
     /* the lookup-producer is the source of the chain */
     if ( rc == 0 )
         rc = execute_lookup_production( tool_ctx -> dir,
+                                        tool_ctx -> vdb_mgr,
                                         tool_ctx -> accession_short,
                                         bg_vec_merger, /* drives the bg_file_merger */
                                         tool_ctx -> cursor_cache,
@@ -751,12 +742,13 @@ static rc_t produce_final_db_output( tool_ctx_t * tool_ctx )
     
     if ( rc == 0 )
         rc = execute_db_join( tool_ctx -> dir,
+                           tool_ctx -> vdb_mgr,
                            tool_ctx -> accession_path,
                            tool_ctx -> accession_short,
                            &stats,
                            &tool_ctx -> lookup_filename[ 0 ],
                            &tool_ctx -> index_filename[ 0 ],
-                           &( tool_ctx -> tmp_id ),
+                           tool_ctx -> temp_dir,
                            registry,
                            tool_ctx -> cursor_cache,
                            tool_ctx -> buf_size,
@@ -774,27 +766,87 @@ static rc_t produce_final_db_output( tool_ctx_t * tool_ctx )
 
     /* STEP 4 : concatenate output-chunks */
     if ( rc == 0 )
-        rc = temp_registry_merge( registry,
-                          tool_ctx -> dir,
-                          tool_ctx -> output_filename,
-                          tool_ctx -> buf_size,
-                          tool_ctx -> show_progress,
-                          tool_ctx -> print_to_stdout,
-                          tool_ctx -> force,
-                          tool_ctx -> compress ); /* temp_registry.c */
+    {
+        if ( tool_ctx -> stdout )
+            rc = temp_registry_to_stdout( registry,
+                                          tool_ctx -> dir,
+                                          tool_ctx -> buf_size ); /* temp_registry.c */
+        else
+            rc = temp_registry_merge( registry,
+                              tool_ctx -> dir,
+                              tool_ctx -> output_filename,
+                              tool_ctx -> buf_size,
+                              tool_ctx -> show_progress,
+                              tool_ctx -> force,
+                              tool_ctx -> compress,
+                              tool_ctx -> append ); /* temp_registry.c */
+    }
 
     /* in case some of the partial results have not been deleted be the concatenator */
     if ( registry != NULL )
         destroy_temp_registry( registry ); /* temp_registry.c */
 
-    if ( rc == 0 && !( tool_ctx -> print_to_stdout ) )
-        print_stats( &stats ); /* helper.c */
+    if ( rc == 0 )
+        print_stats( &stats ); /* above */
 
     return rc;
 }
 
 /* -------------------------------------------------------------------------------------------- */
 
+static bool output_exists_whole( tool_ctx_t * tool_ctx )
+{
+    return file_exists( tool_ctx -> dir, "%s", tool_ctx -> output_filename );
+}
+
+static bool output_exists_idx( tool_ctx_t * tool_ctx, uint32_t idx )
+{
+    bool res = false;
+    SBuffer s_filename;
+    rc_t rc = split_filename_insert_idx( &s_filename, 4096,
+                            tool_ctx -> output_filename, idx ); /* helper.c */
+    if ( rc == 0 )
+    {
+        res = file_exists( tool_ctx -> dir, "%S", &( s_filename . S ) ); /* helper.c */
+        release_SBuffer( &s_filename ); /* helper.c */
+    }
+    return res;
+}
+
+static bool output_exists_split( tool_ctx_t * tool_ctx )
+{
+    bool res = output_exists_whole( tool_ctx );
+    if ( !res ) res = output_exists_idx( tool_ctx, 1 );
+    if ( !res ) res = output_exists_idx( tool_ctx, 2 );    
+    return res;
+}
+
+static rc_t check_output_exits( tool_ctx_t * tool_ctx )
+{
+    rc_t rc = 0;
+    /* check if the output-file(s) do already exist, in case we are not overwriting */    
+    if ( !( tool_ctx -> force ) && !( tool_ctx -> append ) )
+    {
+        bool exists = false;
+        switch( tool_ctx -> fmt )
+        {
+            case ft_unknown             : break;
+            case ft_special             : exists = output_exists_whole( tool_ctx ); break;
+            case ft_whole_spot          : exists = output_exists_whole( tool_ctx ); break;
+            case ft_fastq_split_spot    : exists = output_exists_whole( tool_ctx ); break;
+            case ft_fastq_split_file    : exists = output_exists_split( tool_ctx ); break;
+            case ft_fastq_split_3       : exists = output_exists_split( tool_ctx ); break;
+        }
+        if ( exists )
+        {
+            rc = RC( rcExe, rcFile, rcPacking, rcName, rcExists );
+            ErrMsg( "fasterq-dump.c fastdump_csra() checking ouput-file '%s' -> %R",
+                     tool_ctx -> output_filename, rc );
+        }
+    }
+    return rc;
+}
+
 static rc_t fastdump_csra( tool_ctx_t * tool_ctx )
 {
     rc_t rc = 0;
@@ -802,6 +854,9 @@ static rc_t fastdump_csra( tool_ctx_t * tool_ctx )
     if ( tool_ctx -> show_details )
         rc = show_details( tool_ctx ); /* above */
 
+    if ( rc == 0 )
+        rc = check_output_exits( tool_ctx ); /* above */
+    
     if ( rc == 0 )
         rc = produce_lookup_files( tool_ctx ); /* above */
 
@@ -825,16 +880,20 @@ static rc_t fastdump_table( tool_ctx_t * tool_ctx, const char * tbl_name )
     if ( tool_ctx -> show_details )
         rc = show_details( tool_ctx ); /* above */
 
+    if ( rc == 0 )
+        rc = check_output_exits( tool_ctx ); /* above */
+
     if ( rc == 0 )
         rc = make_temp_registry( &registry, tool_ctx -> cleanup_task ); /* temp_registry.c */
 
     if ( rc == 0 )
         rc = execute_tbl_join( tool_ctx -> dir,
+                           tool_ctx -> vdb_mgr,
                            tool_ctx -> accession_path,
                            tool_ctx -> accession_short,
                            &stats,
                            tbl_name,
-                           &( tool_ctx -> tmp_id ),
+                           tool_ctx -> temp_dir,
                            registry,
                            tool_ctx -> cursor_cache,
                            tool_ctx -> buf_size,
@@ -844,20 +903,27 @@ static rc_t fastdump_table( tool_ctx_t * tool_ctx, const char * tbl_name )
                            & tool_ctx -> join_options ); /* tbl_join.c */
 
     if ( rc == 0 )
-        rc = temp_registry_merge( registry,
-                          tool_ctx -> dir,
-                          tool_ctx -> output_filename,
-                          tool_ctx -> buf_size,
-                          tool_ctx -> show_progress,
-                          tool_ctx -> print_to_stdout,
-                          tool_ctx -> force,
-                          tool_ctx -> compress ); /* temp_registry.c */
-
+    {
+        if ( tool_ctx -> stdout )
+            rc = temp_registry_to_stdout( registry,
+                                          tool_ctx -> dir,
+                                          tool_ctx -> buf_size ); /* temp_registry.c */
+        else
+            rc = temp_registry_merge( registry,
+                              tool_ctx -> dir,
+                              tool_ctx -> output_filename,
+                              tool_ctx -> buf_size,
+                              tool_ctx -> show_progress,
+                              tool_ctx -> force,
+                              tool_ctx -> compress,
+                              tool_ctx -> append ); /* temp_registry.c */
+    }
+    
     if ( registry != NULL )
         destroy_temp_registry( registry ); /* temp_registry.c */
 
-    if ( rc == 0 && !( tool_ctx -> print_to_stdout ) )
-        print_stats( &stats ); /* helper.c */
+    if ( rc == 0 )
+        print_stats( &stats ); /* above */
 
     return rc;
 }
@@ -867,7 +933,8 @@ static const char * consensus_table = "CONSENSUS";
 static const char * get_db_seq_tbl_name( tool_ctx_t * tool_ctx )
 {
     const char * res = tool_ctx -> seq_tbl_name;
-    VNamelist * tables = cmn_get_table_names( tool_ctx -> dir, tool_ctx -> accession_path ); /* cmn_iter.c */
+    VNamelist * tables = cmn_get_table_names( tool_ctx -> dir, tool_ctx -> vdb_mgr,
+                                              tool_ctx -> accession_path ); /* cmn_iter.c */
     if ( tables != NULL )
     {
         int32_t idx;
@@ -883,8 +950,9 @@ static const char * get_db_seq_tbl_name( tool_ctx_t * tool_ctx )
 
 static rc_t perform_tool( tool_ctx_t * tool_ctx )
 {
-    acc_type_t acc_type;
-    rc_t rc = cmn_get_acc_type( tool_ctx -> dir, tool_ctx -> accession_path, &acc_type ); /* cmn_iter.c */
+    acc_type_t acc_type; /* cmn_iter.h */
+    rc_t rc = cmn_get_acc_type( tool_ctx -> dir, tool_ctx -> vdb_mgr,
+                                tool_ctx -> accession_path, &acc_type ); /* cmn_iter.c */
     if ( rc == 0 )
     {
         /* =================================================== */
@@ -898,20 +966,7 @@ static rc_t perform_tool( tool_ctx_t * tool_ctx )
         }
         /* =================================================== */
         
-        if ( tool_ctx -> remove_temp_path )
-        {
-            bool tmp_exists = dir_exists( tool_ctx -> dir, "%s", tool_ctx -> tmp_id . temp_path ); /* helper.c */
-            if ( tmp_exists )
-            {
-                rc_t rc1 = KDirectoryClearDir ( tool_ctx -> dir, true, "%s", tool_ctx -> tmp_id . temp_path );
-                if ( rc1 == 0 )
-                {
-                    tmp_exists = dir_exists( tool_ctx -> dir, "%s", tool_ctx -> tmp_id . temp_path ); /* helper.c */
-                    if ( tmp_exists )
-                        rc1 = KDirectoryRemove ( tool_ctx -> dir, true, "%s", tool_ctx -> tmp_id . temp_path );
-                }
-            }
-        }
+        /* remove_temp_dir( tool_ctx -> temp_dir, tool_ctx -> dir ); */ /* temp_dir.c */
     }
     return rc;
 }
@@ -942,18 +997,11 @@ rc_t CC KMain ( int argc, char *argv [] )
                 rc = populate_tool_ctx( &tool_ctx, args ); /* above */
                 if ( rc == 0 )
                 {
-                    if ( !( tool_ctx . force ) &&
-                         !( tool_ctx . print_to_stdout ) &&
-                         file_exists( tool_ctx . dir, "%s", tool_ctx . output_filename ) )
-                    {
-                        rc = RC( rcExe, rcFile, rcPacking, rcName, rcExists );
-                        ErrMsg( "creating ouput-file '%s' -> %R", tool_ctx . output_filename, rc );
-                    }
-                    else
-                    {
-                        rc = perform_tool( &tool_ctx );     /* above */
-                    }
+                    rc = perform_tool( &tool_ctx );     /* above */
+
                     KDirectoryRelease( tool_ctx . dir );
+                    destroy_temp_dir( tool_ctx . temp_dir ); /* temp_dir.c */
+                    VDBManagerRelease( tool_ctx . vdb_mgr );
                 }
             }
         }
