@@ -237,7 +237,7 @@ static rc_t print_fastq_1_read( join_stats * stats,
         {
             if ( rec -> read . len != rec -> quality . len )
             {
-                ErrMsg( "row #%ld : read.len(%u) != quality.len(%u)\n", row_id,
+                ErrMsg( "row #%ld : read.len(%u) != quality.len(%u) (A)\n", row_id,
                         rec -> read . len, rec -> quality . len );
                 stats -> reads_invalid++;
                 if ( jo -> terminate_on_invalid )
@@ -269,7 +269,7 @@ static rc_t print_fastq_1_read( join_stats * stats,
             {
                 if ( j -> B1 . S . len != rec -> quality . len )
                 {
-                    ErrMsg( "row #%ld : read.len(%u) != quality.len(%u)\n", row_id,
+                    ErrMsg( "row #%ld : read.len(%u) != quality.len(%u) (B)\n", row_id,
                              j -> B1 . S . len, rec -> quality . len );
                     stats -> reads_invalid++;
                     if ( jo -> terminate_on_invalid )
@@ -314,7 +314,7 @@ static rc_t print_fastq_2_reads( join_stats * stats,
             {
                 if ( rec -> read . len != rec -> quality . len )
                 {
-                    ErrMsg( "row #%ld : read.len(%u) != quality.len(%u)\n", row_id,
+                    ErrMsg( "row #%ld : read.len(%u) != quality.len(%u) (C)\n", row_id,
                             rec -> read . len, rec -> quality . len );
                     stats -> reads_invalid++;
                     if ( jo -> terminate_on_invalid )
@@ -341,9 +341,9 @@ static rc_t print_fastq_2_reads( join_stats * stats,
             {
                 if ( join_results_match2( j -> results, &( rec -> read ), &( j -> B2 . S ) ) ) /* join-results.c */
                 {
-                    if ( j -> B2 . S. len != rec -> quality . len )
+                    if ( j -> B2 . S. len + rec -> read . len != rec -> quality . len )
                     {
-                        ErrMsg( "row #%ld : read.len(%u) != quality.len(%u)\n", row_id,
+                        ErrMsg( "row #%ld : read.len(%u) != quality.len(%u) (D)\n", row_id,
                                 j -> B2 . S. len, rec -> quality . len );
                         stats -> reads_invalid++;
                         if ( jo -> terminate_on_invalid )
@@ -378,7 +378,7 @@ static rc_t print_fastq_2_reads( join_stats * stats,
                     uint32_t rl = j -> B1 . S . len + rec -> read . len;
                     if ( rl != rec -> quality . len )
                     {
-                        ErrMsg( "row #%ld : read.len(%u) != quality.len(%u)\n", row_id,
+                        ErrMsg( "row #%ld : read.len(%u) != quality.len(%u) (E)\n", row_id,
                                 rl, rec -> quality . len );
                         stats -> reads_invalid++;
                         if ( jo -> terminate_on_invalid )
@@ -413,7 +413,7 @@ static rc_t print_fastq_2_reads( join_stats * stats,
                     uint32_t rl = j -> B1 . S . len + j -> B2 . S . len;
                     if ( rl != rec -> quality . len )
                     {
-                        ErrMsg( "row #%ld : read.len(%u) != quality.len(%u)\n", row_id,
+                        ErrMsg( "row #%ld : read.len(%u) != quality.len(%u) (F)\n", row_id,
                                 rl, rec -> quality . len );
                         stats -> reads_invalid++;
                         if ( jo -> terminate_on_invalid )
@@ -781,11 +781,12 @@ static rc_t print_fastq_2_reads_splitted( join_stats * stats,
 
 
 static rc_t extract_csra_row_count( KDirectory * dir,
+                                    const VDBManager * vdb_mgr,
                                     const char * accession_path,
                                     size_t cur_cache,
                                     uint64_t * res )
 {
-    cmn_params cp = { dir, accession_path, 0, 0, cur_cache };
+    cmn_params cp = { dir, vdb_mgr, accession_path, 0, 0, cur_cache };
     struct fastq_csra_iter * iter;
     fastq_iter_opt opt = { false, false, false, false }; /* fastq_iter.h */
     rc_t rc = make_fastq_csra_iter( &cp, opt, &iter ); /* fastq_iter.c */
@@ -1056,6 +1057,8 @@ typedef struct join_thread_data
     join_stats stats; /* helper.h */
     
     KDirectory * dir;
+    const VDBManager * vdb_mgr;
+
     const char * accession_path;
     const char * accession_short;
     const char * lookup_filename;
@@ -1097,7 +1100,8 @@ static rc_t CC cmn_thread_func( const KThread * self, void * data )
     if ( rc == 0 && results != NULL )
     {
         join j;
-        cmn_params cp = { jtd -> dir, jtd -> accession_path, jtd -> first_row, jtd -> row_count, jtd -> cur_cache };
+        cmn_params cp = { jtd -> dir, jtd -> vdb_mgr,
+                          jtd -> accession_path, jtd -> first_row, jtd -> row_count, jtd -> cur_cache };
 
         rc = init_join( &cp,
                         results,
@@ -1150,12 +1154,13 @@ static rc_t CC cmn_thread_func( const KThread * self, void * data )
 }
 
 rc_t execute_db_join( KDirectory * dir,
+                    const VDBManager * vdb_mgr,
                     const char * accession_path,
                     const char * accession_short,
                     join_stats * stats,
                     const char * lookup_filename,
                     const char * index_filename,
-                    const tmp_id * tmp_id,
+                    const struct temp_dir * temp_dir,
                     struct temp_registry * registry,
                     size_t cur_cache,
                     size_t buf_size,
@@ -1167,18 +1172,22 @@ rc_t execute_db_join( KDirectory * dir,
     rc_t rc = 0;
     
     if ( show_progress )
+    {
+        KOutHandlerSetStdErr();
         rc = KOutMsg( "join   :" );
-
+        KOutHandlerSetStdOut();
+    }
+    
     if ( rc == 0 )
     {
         uint64_t row_count = 0;
         bool name_column_present, cmp_read_column_present;
         
-        rc = cmn_check_db_column( dir, accession_path, "SEQUENCE", "NAME", &name_column_present ); /* cmn_iter.c */
+        rc = cmn_check_db_column( dir, vdb_mgr, accession_path, "SEQUENCE", "NAME", &name_column_present ); /* cmn_iter.c */
         if ( rc == 0 )
-            rc = cmn_check_db_column( dir, accession_path, "SEQUENCE", "CMP_READ", &cmp_read_column_present ); /* cmn_iter.c */
+            rc = cmn_check_db_column( dir, vdb_mgr, accession_path, "SEQUENCE", "CMP_READ", &cmp_read_column_present ); /* cmn_iter.c */
         
-        rc = extract_csra_row_count( dir, accession_path, cur_cache, &row_count );
+        rc = extract_csra_row_count( dir, vdb_mgr, accession_path, cur_cache, &row_count );
         if ( rc == 0 && row_count > 0 )
         {
             Vector threads;
@@ -1219,6 +1228,7 @@ rc_t execute_db_join( KDirectory * dir,
                 else
                 {
                     jtd -> dir              = dir;
+                    jtd -> vdb_mgr          = vdb_mgr;
                     jtd -> accession_path   = accession_path;
                     jtd -> accession_short  = accession_short;
                     jtd -> lookup_filename  = lookup_filename;
@@ -1234,8 +1244,8 @@ rc_t execute_db_join( KDirectory * dir,
                     jtd -> thread_id        = thread_id;
                     jtd -> cmp_read_present = cmp_read_column_present;
 
-                    rc = make_joined_filename( jtd -> part_file, sizeof jtd -> part_file,
-                                accession_short, tmp_id, thread_id ); /* helper.c */
+                    rc = make_joined_filename( temp_dir, jtd -> part_file, sizeof jtd -> part_file,
+                                               accession_short, thread_id ); /* temp_dir.c */
 
                     if ( rc == 0 )
                     {
